@@ -6,10 +6,12 @@ import traceback
 import socket
 
 from html import escape as html_escape
+from math import ceil
 
 import aiohttp
 from aiohttp import errors, streams, hdrs, helpers
 from aiohttp.log import server_logger
+from aiohttp.helpers import ensure_future
 
 __all__ = ('ServerHttpProtocol',)
 
@@ -26,9 +28,6 @@ DEFAULT_ERROR_MESSAGE = """
   </body>
 </html>"""
 
-ACCESS_LOG_FORMAT = (
-    '%(h)s %(l)s %(u)s %(t)s "%(r)s" %(s)s %(b)s "%(f)s" "%(a)s"')
-
 
 if hasattr(socket, 'SO_KEEPALIVE'):
     def tcp_keepalive(server, transport):
@@ -91,9 +90,7 @@ class ServerHttpProtocol(aiohttp.StreamProtocol):
                  timeout=0,
                  logger=server_logger,
                  access_log=None,
-                 access_log_format=ACCESS_LOG_FORMAT,
-                 host="",
-                 port=0,
+                 access_log_format=helpers.AccessLogger.LOG_FORMAT,
                  debug=False,
                  log=None,
                  **kwargs):
@@ -106,12 +103,14 @@ class ServerHttpProtocol(aiohttp.StreamProtocol):
         self._timeout = timeout  # slow request timeout
         self._loop = loop if loop is not None else asyncio.get_event_loop()
 
-        self.host = host
-        self.port = port
         self.logger = log or logger
         self.debug = debug
         self.access_log = access_log
-        self.access_log_format = access_log_format
+        if access_log:
+            self.access_logger = helpers.AccessLogger(access_log,
+                                                      access_log_format)
+        else:
+            self.access_logger = None
 
     @property
     def keep_alive_timeout(self):
@@ -138,18 +137,20 @@ class ServerHttpProtocol(aiohttp.StreamProtocol):
 
             # use slow request timeout for closing
             # connection_lost cleans timeout handler
-            self._timeout_handle = self._loop.call_later(
-                timeout, self.cancel_slow_request)
+            now = self._loop.time()
+            self._timeout_handle = self._loop.call_at(
+                ceil(now+timeout), self.cancel_slow_request)
 
     def connection_made(self, transport):
         super().connection_made(transport)
 
-        self._request_handler = asyncio.async(self.start(), loop=self._loop)
+        self._request_handler = ensure_future(self.start(), loop=self._loop)
 
         # start slow request timer
         if self._timeout:
-            self._timeout_handle = self._loop.call_later(
-                self._timeout, self.cancel_slow_request)
+            now = self._loop.time()
+            self._timeout_handle = self._loop.call_at(
+                ceil(now+self._timeout), self.cancel_slow_request)
 
         if self._keep_alive_on:
             tcp_keepalive(self, transport)
@@ -187,17 +188,9 @@ class ServerHttpProtocol(aiohttp.StreamProtocol):
         self._keep_alive = val
 
     def log_access(self, message, environ, response, time):
-        if self.access_log and self.access_log_format:
-            try:
-                environ = environ if environ is not None else {}
-                atoms = helpers.SafeAtoms(
-                    helpers.atoms(
-                        message, environ, response, self.transport, time),
-                    getattr(message, 'headers', None),
-                    getattr(response, 'headers', None))
-                self.access_log.info(self.access_log_format % atoms)
-            except:
-                self.logger.error(traceback.format_exc())
+        if self.access_logger:
+            self.access_logger.log(message, environ, response,
+                                   self.transport, time)
 
     def log_debug(self, *args, **kw):
         if self.debug:
@@ -245,8 +238,9 @@ class ServerHttpProtocol(aiohttp.StreamProtocol):
 
                 # start slow request timer
                 if self._timeout and self._timeout_handle is None:
-                    self._timeout_handle = self._loop.call_later(
-                        self._timeout, self.cancel_slow_request)
+                    now = self._loop.time()
+                    self._timeout_handle = self._loop.call_at(
+                        ceil(now+self._timeout), self.cancel_slow_request)
 
                 # read request headers
                 httpstream = reader.set_parser(self._request_parser)
@@ -305,8 +299,10 @@ class ServerHttpProtocol(aiohttp.StreamProtocol):
                         self.log_debug(
                             'Start keep-alive timer for %s sec.',
                             self._keep_alive_period)
-                        self._keep_alive_handle = self._loop.call_later(
-                            self._keep_alive_period, self.transport.close)
+                        now = self._loop.time()
+                        self._keep_alive_handle = self._loop.call_at(
+                            ceil(now+self._keep_alive_period),
+                            self.transport.close)
                     elif self._keep_alive and self._keep_alive_on:
                         # do nothing, rely on kernel or upstream server
                         pass
@@ -363,6 +359,8 @@ class ServerHttpProtocol(aiohttp.StreamProtocol):
             response.send_headers()
 
             response.write(html)
+            # disable CORK, enable NODELAY if needed
+            self.writer.set_tcp_nodelay(True)
             drain = response.write_eof()
 
             self.log_access(message, None, response, self._loop.time() - now)
